@@ -1,9 +1,6 @@
 from collections import defaultdict
 from pyhectiqlab.timer import RepeatedTimer
 
-import threading
-import pyhectiqlab.ops as ops
-
 class Buffer():
 
     def __init__(self, 
@@ -14,6 +11,7 @@ class Buffer():
 
         self.extra_args = extra_args
         self.cache = defaultdict(list)
+        self.new_entries = defaultdict(int)
         self.can_record = defaultdict(lambda: True)
 
         self.max_cache_length = max_cache_length
@@ -33,12 +31,16 @@ class Buffer():
         if self.can_record[key]==False:
             return
 
+        self.new_entries[key] += 1
         self.cache[key].append(value)
         if len(self.cache[key])>self.max_cache_length:
             self.push_data(key)
 
     def push_data(self, key):
-        self.push_method(key, self.cache[key])
+        if self.new_entries[key]>0:
+            self.push_method(key, self.cache[key])
+            
+        self.new_entries[key] = 0 
         self.cache[key] = []
 
     def __delete__(self):
