@@ -2,22 +2,18 @@ from collections import defaultdict
 from pyhectiqlab.timer import RepeatedTimer
 
 import numpy as np
-import threading
-import pyhectiqlab.ops as ops
 import time
 
 class MetricsManager():
 
-    def __init__(self, run_id,
-        max_cache_timeout=5,  # seconds
-        max_cache_length=100, # Number of elements
-        min_cache_flush_delay=5, # Minimum number of seconds between a cache flush
-        aggregate_metrics="none", # Wheter to push all data in cache or aggregate
-        push_method=None):
-        self._run = run_id
+    def __init__(self, max_cache_timeout=10,  # Maximum number of seconds in the cache
+                    max_cache_length=100, # Number of elements in the cache to trigger a cache dump
+                    min_cache_flush_delay=5, # Minimum number of seconds between two dumps
+                    aggregate_metrics="none", # Wheter to push all data in cache or aggregate
+                    push_method=None):
         self.cache = defaultdict(list)
         self.cache_last_push = {}
-
+        self._max_cache_quantity = 1000
         self.max_cache_length = max_cache_length
         self.min_cache_flush_delay  = min_cache_flush_delay
         self.push_method = push_method
@@ -55,10 +51,22 @@ class MetricsManager():
                     return      
             self.push_data(key)
 
+    def subsample(self, data):
+        if len(data)<=self._max_cache_quantity:
+            return data
+        else:
+            print(f"Your metrics rate ({len(data)} metrics/5 seconds) is exceeding the maximum rate ({self._max_cache_quantity}/5 seconds). Your metrics have been subsampled automatically.")
+            print("Consider reducing the number of metrics you push or settings an aggregation method with `run.set_metrics_aggr('mean')`.")
+            print("See documentation for details https://docs.hectiq.ai/objects/metrics/.")
+            indexes = np.arange(0, len(data), len(data)/self._max_cache_quantity).astype(int)
+            return np.array(data)[indexes].tolist()
+
     def push_data(self, key):
         self.cache_last_push[key] = time.time()
+
         if self.aggregate_metrics=="none":
-            self.push_method(key, self.cache[key])
+            data = self.subsample(self.cache[key])
+            self.push_method(key, data)
         elif self.aggregate_metrics=="mean":
             k = [_[0] for _ in self.cache[key]]
             s = [_[1] for _ in self.cache[key]]
@@ -76,7 +84,6 @@ class MetricsManager():
                 self.push_method(key, [(float(np.max(k)), float(np.sum(s)))])
         self.cache[key] = []
 
-
     def __delete__(self):
         self.timer.stop()
         self.flush_cache()
