@@ -1,17 +1,14 @@
 import os
 import sys
 
-import pyhectiqlab
 from pyhectiqlab.metrics import MetricsManager
 from pyhectiqlab.settings import app_url
 from pyhectiqlab.events_manager import EventsManager
 from pyhectiqlab.stream import LogHandler
 from pyhectiqlab.buffer import Buffer
 from pyhectiqlab.watermark import Watermark
-from pyhectiqlab.artifacts import SharedArtifactsManager
 from pyhectiqlab.notebooks import get_notebook_file
-from pyhectiqlab.utils import list_all_files_in_dir
-from pyhectiqlab.decorators import write_method, action_method, beta, will_be_depreciated
+from pyhectiqlab.decorators import write_method, action_method
 
 from pyhectiqlab.mlmodels import download_mlmodel as ops_download_mlmodel
 from pyhectiqlab.mlmodels import upload_mlmodel as ops_upload_mlmodel
@@ -21,128 +18,117 @@ from pyhectiqlab.mlmodels import mlmodel_info_at_path
 from pyhectiqlab.datasets import dataset_info_at_path
 
 from functools import partial
-from typing import Optional
-from sys import getsizeof
-import packaging 
+from typing import Optional, Union
 import tempfile
 import logging
 import shutil
 
 class Run():
     
-    def __init__(self, name: str, project: str = None, should_create_project: bool = False, dry: bool = False):
+    def __init__(self, name: str = None, project: str = None, read_only: bool = False, path: str = None):
+        """Initiate a new run
+        name: Run name. It can contains special caracters.
+            A slug version will be used for reference and slug uniqueness in a project is enforced. 
+        project: Name of the project to connect to (e.g., `hectiqai/dev`).
+        read_only: If True, no data will be pushed to the lab. Use this to prevent from pushing 
+                data in the cloud. In a read only mode, the run is an empty shell where most methods
+                are unexecuted.
         """
-        name: Run name. If a run exists with this name, you'll be attached to the existing run. Otherwise a 
-            new run is created.
-        project: The project name. Leave None if the user with the API key has access to a single project. 
-            Otherwise, you must specify the project name.
-        should_create_project: bool. If true, creates a project d if if does not exists (Admin only).
-        dry: bool: If true, the run is in a dry mode (no writting).
-        
-        """
-        self._mock = dry
-        self.events_manager = EventsManager(self._mock)
-        # Check the python version
-        self.events_manager.compare_python_version()  
-
-        if dry:
-            print("Dry mode activated.")
-            self.run_view = {"id": "null"}
-            self._mock = True
-            self.events_manager = EventsManager(self._mock)
-            self._read_only = False
-        elif self.events_manager.is_logged()==False:
-            print("User not authentificated. Use `pyhectiqlab.login()` for a web prompt or `hectiqlab login` in command line.")
-            self._failed_init()
-        else:
-            # try:
-            if project is None:
-                projects = self.events_manager.add_event("get_all_projects", args=(), auth=True, async_method=False)
-                if len(projects["result"])==1:
-                    project = projects["result"][0]["name"]
-                    print(f"Connecting to project {project}.")
-                    self.project_view = projects["result"][0]
-                else:
-                    print("User has access to multiple projects. Switching to dry mode. \n--------\nFor active mode, please use a project:")
-                    for el in projects["result"]:
-                        print(f"{el['name']}")
-                    self._failed_init()
-                    pass
+        assert project, "The arugment `project` is required."
+        assert name or path, "One of `name` or `path` must be specified."
+        self._read_only = read_only
+        if self._preflight():
+            if path:
+                self.run_view = self.events_manager.add_event('fetch_run', args=(path, project), auth=True, async_method=False)
             else:
-                self.project_view = self.events_manager.add_event("create_project", args=(project, should_create_project), auth=True, async_method=False)
-                if self.project_view is None:
-                    print("Project does not exists. Dry mode activated. \n--------\nUse should_create_project=True to create a project.")
-                    self._failed_init()
-                    pass
-                if self.project_view['status_code']==401:
-                    self._failed_init()
-                    pass
-            if project:
-                self.run_view = self.events_manager.add_event("create_run", args=(name, project), auth=True, async_method=False)
-                try:
-                    if self.run_view["is_new"]==False:
-                        print(f"Attaching to an existing run (ID :{self.run_view['p_id']}).")
-                    if self.run_view['readonly']:
-                        print(f"Run is read only.")
-                    self._read_only = self.run_view['readonly']
-                except:
-                    print("An error occured while creating the run. Switching to dry mode.")
-                    self._failed_init()
+                self.run_view = self.events_manager.add_event('create_run', args=(name, project), auth=True, async_method=False)
+            self._setup_run()
 
-                if "id" not in self.run_view:
-                    print("An error occured while creating the run. Switching to dry mode.")
-                    self._failed_init()
+    @classmethod
+    def existing(cls, path: str, project: str, read_only: bool = False):
+        """Connect to an existing run using the path. The path
+        can be found in web application.
+        
+        Example usage:
+        --------------------
+        run = Run.existing('first-run', project='hectiqlab/dev')
+        """
+        cls = cls(path=path, project=project, read_only=read_only)
+        return cls
 
-        self.name = name
-        self.metrics_manager = MetricsManager(run_id=self.run_view["id"], push_method=self._push_metrics)
-        self.metrics_manager.set_aggr('mean')
-        self.metrics_manager.update_cache_settings(max_cache_timeout=20,
-                                                    min_cache_flush_delay=5, 
-                                                    max_cache_length=25)
-        self.log_recorder = None
-        self.logger_name = ''
-        self.is_recording_logs = False
+    def _preflight(self):
+        """Check if user is logged in. Do the correct setup if read only mode.
+        """
+        self.events_manager = EventsManager(self._read_only)
+        self.events_manager.compare_python_version()
+        
+        self.log_streams = {}
+        self._stream_exception()
+
+        if self._read_only:
+            print('Read only mode.')
+            self.setup_read_only()
+            return False
+        
+        if self.events_manager.is_logged()==False:
+            self.user_is_not_logged()
+            return False
+
+        return True
+
+    def _setup_run(self):
+        try:
+            assert self.run_view.get('status_code')==200, 'An error occured.'
+
+            if self.run_view.get("is_new", False)==False:
+                print(f"Attaching to an existing run (ID :{self.run_view.get('p_id')}).")
+
+            if self.run_view.get('readonly', True):
+                print("Warning::Run is read only because you are not the author of the run.")
+                self._read_only = True
+                self.setup_read_only()
+
+            if self.run_view.get('slug', None) is None:
+                print("An error occured while creating the run. Switching to read only mode.")
+                if self.run_view.get('detail'):
+                    print("Error: ", self.run_view.get('detail'))
+                self.setup_read_only()
+                return
+        except:
+            print("An error occured while creating the run. Switching to read only mode.")
+            if self.run_view.get('detail'):
+                print("Error: ", self.run_view.get('detail'))
+            self.setup_read_only()
+
+        self.metrics_manager = MetricsManager(push_method=self._push_metrics)
         self.logs_buffer = Buffer(push_method=self._push_logs)
-        return
 
-    def set_metrics_cache_settings(self, aggr: str = 'mean', max_cache_timeout=None, max_cache_length=None, min_cache_flush_delay=None):
-        """
-        Update the settings of the metrics manager.
+    def user_is_not_logged(self):
+        print("User not authentificated. Use `hectiqlab add-profile` in command line.")
+        self.setup_read_only()
 
-        max_cache_timeout [int]: Time interval for flushing the cache (seconds)  (default is 20)
-        max_cache_length [int]: Max number of elements before flushing the cache (default is 25)
-        min_cache_flush_delay [float]: Minimum delay between to successive data send (for the same key). 
-                If the number of matrix is added faster than this delay, the cache will increase to
-                respect the delay. (default is 5 seconds)
-        aggr [str]: The aggregate method. One of ['none', 'sum', 'max', 'mean']  (default is mean)
-        """
-        self.metrics_manager.set_aggr(aggr)
-        self.metrics_manager.update_cache_settings(max_cache_timeout=max_cache_timeout, 
-                                                    max_cache_length=max_cache_length,
-                                                    min_cache_flush_delay=min_cache_flush_delay)
-
-    def _failed_init(self):
-        self.run_view = {"id": "null"}
-        self._mock = True
-        self.events_manager = EventsManager(self._mock)
+    def setup_read_only(self):
         self._read_only = True
-
-    @will_be_depreciated('add_artifact')
-    def add_artifacts(self, *args, **kwargs):
-        self.add_artifact(*args, **kwargs)
+        self.run_view = {}
+        self.events_manager = EventsManager(True)
+        self.metrics_manager = None
+        self.logs_buffer = None
 
     @write_method
     @action_method
-    def add_artifact(self, filepath: str, step: int = None, wait_response: bool = False):
+    def add_artifact(self, filepath: str, name: str = None, step: int = None, wait_response: bool = False):
         """Log a file as an artifacts. If a file already exists with the name, it will be overwritten.
 
         filepath [str] : Path to the file.
+        name [str]: Use this to overwrite the name of the artifact. If None, the basename of the filepath is used.
+                    Use this especially if you are using step artifacts and you want to group multiple files
+                    with different names.
         step [int]: The optional step stamp of the artifacts.
         wait_response [bool]: Set to true to upload sync. If False, the upload is made in background.
         """
         filename = os.path.basename(filepath)
         num_bytes = os.path.getsize(filepath)
-        args = (self._id, filename, filepath, num_bytes, step)
+        args = (self._id, self._project_path, filename, filepath, name, num_bytes, step)
         return self.events_manager.add_event("add_artifact", args, async_method=bool(1-wait_response))
 
     @write_method
@@ -154,7 +140,7 @@ class Run():
         mlmodel_name: Name of the mlmodel
         version: Version of the mlmodel. If None, the latest version is used.
         """
-        args = (self._id, mlmodel_name, version, self.project_view['id'])
+        args = (self._id, self._project_path, mlmodel_name, version)
         self.events_manager.add_event("log_mlmodel", args)
 
     @write_method
@@ -170,7 +156,7 @@ class Run():
         assert meta is not None, f'Could not find the mlmodel at {dirpath}'
 
         print(f"{meta.get('name')} - {meta.get('version')}")
-        args = (self._id, meta.get('name'), meta.get('version'), self.project_view['id'])
+        args = (self._id, self._project_path, meta.get('name'), meta.get('version'))
         self.events_manager.add_event("log_mlmodel", args)
         return
         
@@ -190,13 +176,15 @@ class Run():
         """
         mlmodel = ops_upload_mlmodel(source_path=source_path, 
             mlmodel_name=name,
-             run_id=self._id, 
+             run_id=self._id,
+             project_path=self._project_path,
              version=version,
              short_description=short_description,
              push_dir=push_dir)
         if mlmodel is not None:
             self.add_mlmodel_usage(mlmodel.get('name'), version=mlmodel.get('version'))
 
+    @write_method
     @action_method
     def download_mlmodel(self, mlmodel_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
         """Download an existing mlmodel from the run's project.
@@ -208,7 +196,7 @@ class Run():
         overwrite: Set to True to overwrite in save_path.
         """
         dirpath = ops_download_mlmodel(mlmodel_name=mlmodel_name, 
-                        project_id=self.project_view["id"], 
+                        project_path=self._project_path, 
                         version=version, 
                         save_path=save_path, 
                         overwrite=overwrite)
@@ -233,6 +221,7 @@ class Run():
         dataset = ops_upload_dataset(source_path=source_path, 
             dataset_name=name,
              run_id=self._id, 
+             project_path=self._project_path,
              version=version,
              short_description=short_description,
              push_dir=push_dir)
@@ -249,7 +238,7 @@ class Run():
         version: Version of the dataset. If None, the latest version is used.
 
         """
-        args = (self._id, dataset_name, version, self.project_view['id'])
+        args = (self._id, self._project_path, dataset_name, version)
         self.events_manager.add_event("log_dataset", args)
 
     @write_method
@@ -264,10 +253,11 @@ class Run():
         assert meta is not None, f'Could not find the dataset at {dirpath}'
 
         print(f"{meta.get('name')} - {meta.get('version')}")
-        args = (self._id, meta.get('name'), meta.get('version'), self.project_view['id'])
+        args = (self._id, self._project_path, meta.get('name'), meta.get('version'))
         self.events_manager.add_event("log_dataset", args)
         return
 
+    @write_method
     @action_method
     def download_dataset(self, dataset_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
         """Download an existing dataset from the run's project.
@@ -279,7 +269,7 @@ class Run():
         overwrite: Set to True to overwrite in save_path.
         """
         dirpath = ops_download_dataset(dataset_name=dataset_name, 
-                        project_id=self.project_view["id"], 
+                        project_path=self._project_path, 
                         version=version, 
                         save_path=save_path, 
                         overwrite=overwrite)
@@ -289,20 +279,15 @@ class Run():
     
     @write_method
     @action_method
-    def add_config(self, config: 'Config', prefix=""):
-        """Log a config object (pyhectiqlab.Config). Use prefix 
-        to log a config in a nested path.
+    def add_config(self, config: Union['pyhectiqlab.Config', dict]):
         """
-        config.push_as_meta(self, prefix=os.path.join("config/", prefix))
-
-    @write_method
-    @action_method
-    @will_be_depreciated('add_tf_model_as_artifact')
-    def add_tf_model(self, *args, **kwargs):
-        """Add a tensorflow model as an artifact. This method will be depreciated. Use 
-        `add_tf_model_as_artifact`
+        Add a config object of a dicté
         """
-        self.add_tf_model_as_artifact(*args, **kwargs)
+        if isinstance(config, dict):
+            args = (self._id, self._project_path, config)
+        else:
+            args = (self._id, self._project_path, config.dict)
+        self.events_manager.add_event("push_config", args)
 
     @write_method
     @action_method
@@ -327,30 +312,29 @@ class Run():
 
     @write_method
     @action_method
-    def add_shared_artifact(self, filename: str):
-        """Push a shared artifact.
+    def set_metrics_aggr(self, aggr: 'str' = "none"):
         """
-        m = SharedArtifactsManager()
-        m.push_artifact(filepath=filename, project_id=self.project_view["id"])
+        Set an aggregation method for the metrics. 
 
-    def list_shared_artifact(self):
-        """Returns a list of the shared artifacts for this project.
+        aggr [str]: The aggregate method. One of ['none', 'sum', 'max', 'mean']  (default is none)
+                    If 'none', all metrics are pushed to the lab. Otherwise, only the aggregated value is
+                    pushed. If you plan to exceed the maximum rate (1000 metrics/5 seconds), it is best to use an aggregation method.
         """
-        m = SharedArtifactsManager()
-        return m.list_artifacts(project_id=self.project_view["id"])
+        assert aggr in ['none', 'sum', 'max', 'mean']
+        self.metrics_manager.set_aggr(aggr)
 
     @write_method
     @action_method
-    def add_metrics(self, key: 'str', value: float, step: int):
+    def add_metrics(self, key: 'str', value: float, step: float):
         """Add a metrics. The metrics is saved even if there already exists a value
         for the same key and step. 
 
         key [str]: Key used for the metrics. The format with backlash such as `metrics/train`
             will be recognized to organize the metrics.
         value [float]. If tensor, is casted to float.
-        step [int]: Training step. Casted to integer.
+        step [float]: Step
         """
-        self.metrics_manager.add(key, float(value), int(step))
+        self.metrics_manager.add(key, float(value), step)
 
     @write_method
     @action_method
@@ -358,7 +342,7 @@ class Run():
         """Add tag to a run. If the tag does not already exists by this name,
         a new tag is created with the given description and color.
         """
-        args = (self._id, name, description, color)
+        args = (self._id, self._project_path, name, description, color)
         self.events_manager.add_event("add_tag", args)
 
     @write_method
@@ -382,7 +366,7 @@ class Run():
             if git_info:
                 versions["git"] = water.get_git_info()
 
-        args = (self._id, versions)
+        args = (self._id, self._project_path, versions)
         self.events_manager.add_event("push_package_versions", args)
 
     @write_method
@@ -397,35 +381,36 @@ class Run():
         water = Watermark()
         result = water.get_repo_info(package)
         data = {package: result}
-        args = (self._id, data)
+        args = (self._id, self._project_path, data)
         self.events_manager.add_event("push_git_package_state", args)
 
     @write_method
     @action_method
-    def add_current_notebook(self, stamp: str = 'datetime'):
+    def add_current_notebook(self, name: str = None, stamp: str = 'datetime'):
         """If the run is executed in a jupyter notebook, this
         will add your current notebook as an artifact.
-
+        
+        name [str] : Name of the artifact. Optional.
         stamp [str]: Add a stamp on the notebook name.
                     One of 'datetime', 'date', None
         """
-        filename = get_notebook_file(stamp)
-        self.add_artifact(filename)
+        path, fname = get_notebook_file(stamp)
+        self.add_artifact(path, name=fname or name)
 
     @write_method
     @action_method
     def failed(self):
         """Set the run to the failed status.
         """
-        args = (self._id, "failed")
+        args = (self._id, self._project_path, "failed")
         self.events_manager.add_event("set_run_status", args)
 
     @write_method
     @action_method
-    def _stopped(self):
+    def stopped(self):
         """Set the run to the stopped status.
         """
-        args = (self._id, "stopped")
+        args = (self._id, self._project_path, "stopped")
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
@@ -433,7 +418,7 @@ class Run():
     def completed(self):
         """Set the run to the completed with success status.
         """
-        args = (self._id, "completed")
+        args = (self._id, self._project_path, "completed")
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
@@ -441,7 +426,7 @@ class Run():
     def pending(self):
         """Set the run to the pending status.
         """
-        args = (self._id, "pending")
+        args = (self._id, self._project_path, "pending")
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
@@ -449,7 +434,7 @@ class Run():
     def running(self):
         """Set the run to the running status.
         """
-        args = (self._id, "running")
+        args = (self._id, self._project_path, "running")
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
@@ -457,7 +442,7 @@ class Run():
     def training(self):
         """Set the run to the training status.
         """
-        args = (self._id, "training")
+        args = (self._id, self._project_path, "training")
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
@@ -466,7 +451,7 @@ class Run():
         """Add or update a meta information in the format of
         key/value.
         """ 
-        args = (self._id, key, value)
+        args = (self._id, self._project_path, key, value)
         self.events_manager.add_event("push_meta", args)
         
     @write_method
@@ -474,7 +459,7 @@ class Run():
     def set_note(self, text: str):
         """Add or update a custom note field for the run.
         """
-        args = (self._id, text)
+        args = (self._id, self._project_path, text)
         self.events_manager.add_event("set_note", args)
     
     @write_method
@@ -482,80 +467,96 @@ class Run():
     def set_paper(self, content: str):
         """Overwrite the content of the paper of the run.
         """
-        args = (self._id, content)
+        args = (self._id, self._project_path, content)
         self.events_manager.add_event("set_paper", args)
 
-    @write_method
-    @action_method
-    def start_recording_logs(self, logger_name=None):
+    def _stream_exception(self):
+        """
+        Handle the exception with `sys.excepthook`.
+        """
+        def handle_exception(exc_type, exc_value, exc_traceback):
+            if issubclass(exc_type, KeyboardInterrupt):
+                self.stopped()
+            else:
+                self.add_tag(name="failed")
+                self.failed()
+            
+            logger = self.add_log_stream(level=50, format="%(asctime)s - %(levelname)s - %(message)s")
+            logger.critical("Automatic exception handler", exc_info=(exc_type, exc_value, exc_traceback))
+
+            # Flush cache before leaving
+            self.logs_buffer.flush_cache()
+            self.metrics_manager.flush_cache()
+            sys.exit(0)                
+            
+        sys.excepthook = handle_exception
+        return 
+
+    def add_log_stream(self, logger_name: Optional[str] = None, 
+                        format: Optional[Union[str, "logging.Formatter"]] = "%(asctime)s - %(levelname)s - %(message)s", 
+                        level: Optional[int] = 1):
         """Start tailing the logs on a logger name.
         If logger_name is None, the default __name__ will be taken and
         you may use the returned logger.
         """
-        self.is_recording_logs = True
-        self.logs_buffer.start(key="logs")
+        if self.dry_mode or self.read_only:
+            print('A logger is returned but the logs are not stored because you are in a dry (offline) mode.')
+            return logging.getLogger(logger_name)
         
-        print("Start recording logs.")
-        if (self.log_recorder is None) and self.logger_name!=logger_name:
-            func = partial(self.logs_buffer.add, key="logs")
-            self.log_recorder = LogHandler(func)
-            console  = logging.StreamHandler()  
-            console.setLevel(logging.DEBUG)
+        # Start recording
+        self.logs_buffer.start(key=logger_name)
 
-            name = logger_name
-            if name is None:
-                name = __name__
-            self.logger_name = name
+        if logger_name not in self.log_streams:
+            # Add log
+            func = partial(self.logs_buffer.add, key=logger_name)
+            log_recorder = LogHandler(func)
+            log_recorder.set_name("hectiqlab")
+            console = logging.StreamHandler()
+            console.setLevel(level)
+            logger = logging.getLogger(logger_name)
+            logger.setLevel(level)
+            logger.addHandler(log_recorder)
+            if isinstance(format, str):
+                formatter = logging.Formatter(format)
+            else:
+                formatter = format
+            log_recorder.setFormatter(formatter)
+            self.log_streams[logger_name] = logger
 
-            logger = logging.getLogger(name)
-            logger.setLevel(logging.DEBUG)
-            logger.addHandler(self.log_recorder)
+        return self.log_streams[logger_name]
 
-            # Handle all exceptions
-            def handle_exception(exc_type, exc_value, exc_traceback):
-                if issubclass(exc_type, KeyboardInterrupt):
-                    self._stopped()
-                else:
-                    self.add_tag(name="failed")
-                    self.failed()
-                    
-                logger.error("Exception", exc_info=(exc_type, exc_value, exc_traceback))
-                # Flush cache before leaving
-                self.logs_buffer.flush_cache()
-                self.metrics_manager.flush_cache()
-                sys.exit(0)                
-                
-            sys.excepthook = handle_exception
-
-            return logger
-        else:
-            return logging.getLogger(self.logger_name)
-        
-    @write_method
-    @action_method
-    def stop_recording_logs(self):
-        """Stop recording the logs from the logger. 
+    def remove_log_stream(self, logger_name: Optional[str] = None):
+        """Stop recording a certain log stream.
         """
-        self.is_recording_logs = False
-        self.logs_buffer.stop(key="logs")
-        
+        self.logs_buffer.stop(key=logger_name)
+        if logger_name in self.log_streams:
+            logger = self.log_streams[logger_name]
+            for handler in logger.handlers:
+                if handler.get_name()=="hectiqlab":
+                    logger.removeHandler(handler)
+
+        self.log_streams.pop(logger_name, None)
+
     @write_method
     @action_method
     def _push_logs(self, key, msg):
-        if key=="logs":
-            if self.is_recording_logs:
-                args = (self._id, "".join(msg))
-                self.events_manager.add_event("append_logs", args)
+        args = (self._id, self._project_path, "".join(msg))
+        self.events_manager.add_event("append_logs", args)
         
     @write_method
     @action_method
     def _push_metrics(self, key, values):
-        args = (self._id, key, values)
-        self.events_manager.add_event("push_metrics", args)
+        if len(values)>0:
+            args = (self._id, self._project_path, key, values)
+            self.events_manager.add_event("push_metrics", args)
 
     @property
     def _id(self):
-        return self.run_view["id"]
+        return self.run_view.get('slug')
+
+    @property
+    def _project_path(self):
+        return self.run_view.get('project', {}).get('path', '')
 
     @property
     def read_only(self):
@@ -563,7 +564,7 @@ class Run():
 
     @property
     def dry_mode(self):
-        return self._mock
+        return self._read_only
 
     @property
     def action_mode(self):
@@ -583,9 +584,11 @@ class Run():
     def __str__(self):
         # print(self.run_view)
         pad = 9
-        path = f"/{self.project_view['id']}/runs/{self._id}"
-        return f"<Run {self.run_view['p_id']}>"\
-                f"\n{'project'.ljust(pad)}: {self.project_view['name']}"\
+        project = self.run_view.get('project', {})
+        path = f"/{project.get('path')}/runs/{self.run_view.get('slug')}"
+        return f"<Run {self.run_view.get('p_id')} {self.run_view.get('slug')}>"\
+                f"\n{'Name'.ljust(pad)}: {self.run_view.get('name')}"\
+                f"\n{'Project'.ljust(pad)}: {project.get('name')} ({project.get('path')})"\
                 f"\n{'author'.ljust(pad)}: {self.author}"\
                 f"\n{'mode'.ljust(pad)}: {self.action_mode}"\
                 f"\n{'url'.ljust(pad)}: {app_url+path} "
